%-------------------------------------------------------------------------------------------------------
% PURPOSE: "Backtesting Marginal Expected Shortfall and Related Systemic Risk Measures"
%           Banulescu, Leymarie, Hurlin and Scaillet (2016)
%-------------------------------------------------------------------------------------------------------
%   H1 : Normal Static Estimation with Normal GJRDCC DGP
%-------------------------------------------------------------------------------------------------------
% Note: The calibration of the parameters is obtained from the estimation of a GJR-DCC model on BAC and 
%       CRSP market weighted index (2012-2016 included)
%-------------------------------------------------------------------------------------------------------
% J. Leymarie, November 28, 2017.
% LEO, University of Orleans
%-------------------------------------------------------------------------------------------------------

clear ; clc ; close all ;  tic

help Main_Power_Conditional_MES_H1_Marginal_MES

load('C:\Users\jeremy\Desktop\Monte_Carlo\Conditional_MES\Size_Conditional_MES_forpowersizecorrected')

%==================%
%=== Parameters ===%
%==================%

cases=[250 250 ; 250 500 ; 500 250 ; 500 500; 2500 250; 2500 500]; % Sample sizes (first colum T, second coum N)

S=10;                                            % Number of replications

alpha=0.05;                                        % alpha level for quantile of Y2

level=0.05;                                        % Nominal size of the tests

mmax=5;                                            % Maximum lag autocovariance for IND test

options=optimset('Display','off');                 % Options

%======================================================%
%=== Variance-Covariance and Correlation Parameters ===%
%======================================================%

a1=[0.149;0.065;0.075;0.855];                      % GARCH parameters for Y1 (GJR-DCC)

a2=[0.057;0.001;0.268;0.786];                      % GARCH parameters for Y2 (GJR-DCC)
                 
b=[0.663;0.041;0.754];                             % DCC parameters (rho_bar,a,b) (Normal DCC) 
                         
var1=a1(1)/(1-a1(2)-0.5*a1(3)-a1(4));              % Unconditional variance for Y1

var2=a2(1)/(1-a2(2)-0.5*a2(3)-a2(4));              % Unconditional variance for Y2

corr12=b(1);                                       % Unconditional correlation (Normal DCC)

cov12=corr12*sqrt(var1)*sqrt(var2);                % Unconditional covariance

Q_bar=[1 b(1);b(1) 1];                             % Unconditionnal correlation matrix 

theta0=[a1'  a2'  b'];                             % True value of the parameters

np=length(theta0);                                 % Number of estimated parameters

%======================%
%=== Initialization ===%
%======================%

ncases=size(cases,1);                              % Number of considered cases

uc=NaN(S,ncases); ind=NaN(S,mmax,ncases);          % UC and IND statistics without estimation risk

uc_risk=uc; ind_risk=ind;                          % UC and IND statistics with estimation risk

uc_c=uc; uc_c_kernel=uc;                           % UC robust statistics

ind_c=ind; ind_c_kernel=ind;                       % IND robust statistics

p_uc=uc; p_ind=ind;                                % UC and IND p_values without estimation risk

p_uc_risk=uc; p_ind_risk=ind;                      % UC and IND p_values with estimation risk

p_uc_c=uc;p_uc_c_kernel=uc;                        % UC robust p_values

p_ind_c=ind;p_ind_c_kernel=ind;                    % IND robust p_values 

for cas=1:ncases

    T=cases(cas,1);                                % In sample size

    N=cases(cas,2);                                % Out of sample size

    TT=T+N;                                        % Total size
    
%============================
%=== Simulations under H0 ===
%============================

parfor s=1:S                                       % Loop for the simulations
    
% if floor(s/100)==(s/100) ; disp(s); end          % Display the simulation number
        
Y=NaN(TT+500,2);Y(1,:)=0;                          % Initialization of Y1,Y2 series 

Sigma_t=NaN(2,2,TT+500);                           % Initialization of the cond. variance matrices

rho=NaN(TT+500,1);rho(1)=corr12;                   % Initialization of cond. correlations

S2=NaN(TT+500,2);S2(1,1)=var1;S2(1,2)=var2;        % Initialization of the cond. variances

Qlag=Q_bar;                                        % Lagged cond. correlation matrix

z=mvnrnd(zeros(2,1),eye(2),TT+500);                % Simulated innovations                                                                              
                                                            
for t=2:TT+500                                                  % Loop for time t
    
    S2(t,1)=a1(1)+a1(2)*Y(t-1,1)^2+...                          % Conditionnal variance of Y1 at time t
        a1(3)*(Y(t-1,1)<0)*Y(t-1,1).^2+...
        a1(4)*S2(t-1,1); 
    
    S2(t,2)=a2(1)+a2(2)*Y(t-1,2)^2+...                          % Conditionnal variance of Y2 at time t
        a2(3)*(Y(t-1,2)<0)*Y(t-1,2)^2+...
        a2(4)*S2(t-1,2);  
    
    eps=Y(t-1,:)./sqrt(S2(t-1,:));                              % Standardized Y1 Y2 at time t-1
    
    Q=(1-b(2)-b(3))*Q_bar+b(2)*(eps'*eps)+b(3)*Qlag;            % Q matrix at time t
    
    Qlag=Q;                                                     % Qlag becomes Q
    
    R=diag(diag(1./sqrt(Q)))*Q*diag(diag(1./sqrt(Q)));          % R matrix at time t
    
    rho(t)=R(1,2);                                              % Extraction of conditionnal correlation between Y1 Y2
   
    Sigma_t(:,:,t)=sqrt(diag(S2(t,:)))*R*sqrt(diag(S2(t,:)));   % Conditional covariance matrix at time t

    Y(t,:)=z(t,:)*chol(Sigma_t(:,:,t));                         % Simulated series

end                                                             % Loop on t
           
Y=Y(501:end,:);                                                 % Removing 500 first values of Y

%===============%
%=== Figures ===%
%===============%

% plot([1:TT],Y(:,1),[1:TT],Y(:,2)),legend('Y1','Y2')
% plot([1:TT],Y(:,2),[1:TT],VaR2(:,:)),legend('Y2 return','Value at Risk at 5%'), xlabel('Time')
% plot([1:TT],S2(:,1),[1:TT],S2(:,2)),xlabel('Time'),legend('Variance cond. Y1','Variance cond. Y2') 
% plot(rho),legend('Correlation between Y1 Y2')

%========================
%=== Estimation In-Sample ===
%========================

Sigma_in=cov(Y(1:T,:),1);                                           % Estimated unconditional variance-covariance matrix 

var1_in=Sigma_in(1,1);                                              % Estimated unconditional variance for Y1

var2_in=Sigma_in(2,2);                                              % Estimated unconditional variance for Y1

rho_in=Sigma_in(1,2)/sqrt(var2_in*var1_in);                         % In sample correlation

cov_in=Sigma_in(1,2);                                               % In sample covariance

VaR2_in=sqrt(var2_in)*norminv(alpha);                               % Estimated Value-at-Risk(alpha ) for Y2

theta_in=[var1_in rho_in var2_in]';                                 % Estimated parameters in-sample

Hessian=hessian_2sided('loglikelihood_normal',theta_in,Y(1:T,:));   % Hessian matrix (Loglik is the opposite of the likelihood)

V_asy=pinv(Hessian);                                                % Asymptotic variance-covariance of ML estimators

%==========================
%=== Forecasts out-of-sample ===
%==========================

var1_out=var1_in;                                % Unconditional variances forecasts for Y1

var2_out=var2_in;                                % Unconditional variances forecasts for Y2

rho_out=rho_in;                                  % Unconditional correlation forecast 

Sigma_out=Sigma_in;                              % Unconditional variance covariance matrix

VaR2_out=VaR2_in;                                % Unconditional VaR forecasts

Y1_out=Y(T+1:end,1);                             % OOS simulated realisations for Y1

Y2_out=Y(T+1:end,2);                             % OOS simulated realisations for Y2

Y_out=[Y1_out Y2_out];                           % OOS simulated realisations for [Y1 Y2]

%==============================================
%=== Computation of H(alpha) series: with estimation risk  ===
%==============================================

Y1_out_risk=Y1_out(Y2_out<=VaR2_out);   % Value of Y1 for which Y<VaR2_out           

u12=mvncdf([Y1_out_risk VaR2_out*ones(size(Y1_out_risk))],0,Sigma_out)/alpha;

H_risk=zeros(N,1);                      % Ht(alpha) series

H_risk(Y2_out<=VaR2_out)=1-u12;         % Ht(alpha) series (evaluated for Y1 for which Y2<VaR2

%=================================
%== UC_MES - Unconditionnal backtest ===  
%=================================

% With Estimation Risk

uc_risk(s,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4));   % Statistic UC with estimation risk

p_uc_risk(s,cas)=2*normcdf(-abs(uc_risk(s,cas)));                          % UC p_value

% Robust Test Statistics

RMESt_kernel=H_normal_derivatives_kernel(Sigma_out,Y_out,alpha,1/N);       % Correction term RMES (H kernel numerical derivative)

RMES_kernel=mean(RMESt_kernel,2);                                          % RMES mean

uc_c_kernel(s,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)+...   % Statistic UC robust
N/T*RMES_kernel'*T*V_asy*RMES_kernel);   

p_uc_c_kernel(s,cas)=2*normcdf(-abs(uc_c_kernel(s,cas)));                 % UC p_value

% RMES=[RMES_CoVaR_term(var1_out,var2_out,rho_out,VaR2_out,1,alpha);...   % Correction term RMES (CoVaR methodology)
%    RMES_CoVaR_term(var1_out,var2_out,rho_out,VaR2_out,2,alpha);...
%    RMES_VaR2_term(var1_out,var2_out,rho_out,VaR2_out,3,alpha)];
% 
% uc_c(s,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)+... % Statistic UC robust
%     N/T*RMES'*T*V_asy*RMES);                             
% 
% p_uc_c(s,cas)=2*normcdf(-abs(uc_c(s,cas)));                             % UC p_value

%=================================
%=== IND_MES - Conditionnal backtest  ===
%=================================

gammaHt_hat=NaN(mmax,1);gammaHt_hat_risk=gammaHt_hat;                                       % Autocovariances of Ht(alpha)

rhoHt_hat=NaN(mmax,1);rhoHt_hat_risk=rhoHt_hat;                                                             % Autocorrelations of Ht(alpha)

R_kernel=NaN(mmax,3);                                                                                                          % Correction term R (Ht kernel numerical derivative)  

% R=NaN(mmax,3);                                                                                                                      % Correction term R (CoVaR methodology)                            

for m=1:mmax                                                                                                                            % Loop on the lag

    % Autocorrelations with and without estimation risk

    Hlag_risk=[NaN(m,1) ; H_risk(1:N-m)];                                                     % Lagged values with Estimation Risk

   gammaHt_hat_risk(m)=(1/(N-m))*nansum((H_risk-alpha/2).*(Hlag_risk-alpha/2));               % Autocovariances of H(t) with Estimation Risk

   rhoHt_hat_risk(m)=gammaHt_hat_risk(m)/var(H_risk);                                         % Autocorrelations of H(t) with Estimation Risk

    % Matrix Ri and Rj 

    R_kernel(m,:)=(1/(alpha*(1/3-alpha/4)))*(nansum(repmat((Hlag_risk-alpha/2),1,3).*RMESt_kernel')/(N-m));       % Be careful: RMES must be a vector in the conditional case, do not use repmat

%     R(m,:)=(1/(alpha*(1/3-alpha/4)))*(nansum(repmat((Hlag_risk-alpha/2),1,3).*repmat(RMES',N,1))/(N-m));        % Be careful: RMES must be a vector in the conditional case, do not use repmat

    % With estimation risk

    ind_risk(s,m,cas)=N*sum(rhoHt_hat_risk(1:m).^2);                                         % Statistic IND with estimation risk 

    p_ind_risk(s,m,cas)=1-chi2cdf(ind_risk(s,m,cas),m);                                      % IND p-value

    % Robust test statistics

%     delta=eye(m)+N*R(1:m,:)*V_asy*R(1:m,:)';                                               % Variance-covariance matrix of the corrected statistic

    delta_kernel=eye(m)+N*R_kernel(1:m,:)*V_asy*R_kernel(1:m,:)';                            % Variance-covariance matrix of the corrected statistic (kernel)

    ind_c_kernel(s,m,cas)=N*rhoHt_hat_risk(1:m)'*pinv(delta_kernel)*rhoHt_hat_risk(1:m);     % Statistic IND robust (Ht kernel numerical derivative)

    p_ind_c_kernel(s,m,cas)=1-chi2cdf(ind_c_kernel(s,m,cas),m);                              % IND p-value

%     ind_c(s,m,cas)=N*rhoHt_hat_risk(1:m)'*pinv(delta)*rhoHt_hat_risk(1:m);                 % Statistic IND robust (CoVaR methodology)
% 
%     p_ind_c(s,m,cas)=1-chi2cdf(ind_c(s,m,cas),m);                                          % IND p-value

end


end                                                    % Loop on S

disp('==================================================================================='),disp(' ')

fprintf('  In-sample size = %1.0f    Out-of-sample size = %1.0f   Number of replications = %1.0f \n',T,N,S),disp(' ')

fprintf('  Coverage rate for VaR = %1.3f      Size = %1.3f \n',alpha,level),disp(' ')

fprintf('  Max lag (for IND test) = %1.0f  \n',mmax)

disp(' '),disp('  ==== Test UC ====')

disp(' '),fprintf('  Empirical power (with ER) = %1.4f \n',mean(abs(uc_risk(:,cas))>=quantile(abs(uc_risk_H0(:,cas)),1-level)))      

disp(' '),fprintf('  Empirical power (robust, Kernel method) = %1.4f \n',mean(abs(uc_c_kernel(:,cas))>=quantile(abs(uc_c_kernel_H0(:,cas)),1-level)))  

disp(' '), disp('  ==== Test IND ====')

disp(' '),disp('  Lags ')  

disp(sprintf('   %1.4f ',(1:1:mmax)) )

disp(' ');disp('  Empirical power (with ER)')

disp(sprintf('   %1.4f ',mean(abs(ind_risk(:,1:mmax,cas))>=repmat(quantile(abs(ind_risk_H0(:,1:mmax,cas)),1-level),S,1))))   

disp(' ');disp('  Empirical power (robust, Kernel method)')

disp(sprintf('   %1.4f ',mean(abs(ind_c_kernel(:,1:mmax,cas))>=repmat(quantile(abs(ind_c_kernel_H0(:,1:mmax,cas)),1-level),S,1))))   

disp(' ')

end

toc

