%-------------------------------------------------------------------------------------------------------
% PURPOSE: "Backtesting Marginal Expected Shortfall and Related Systemic Risk Measures"
%           Banulescu, Leymarie, Hurlin and Scaillet (2016)
%-------------------------------------------------------------------------------------------------------
% Backtesting test for the daily MES and daily SRISK:
% Empirical size got from Monte Carlo simulations based on a Gaussian GJR-DCC model with estimation risk 
% (DCC estimation inside the code) 
%-------------------------------------------------------------------------------------------------------
% Note: The calibration of the parameters is obtained from the estimation of a GJR-DCC model on BAC and 
%       CRSP market weighted index (2012-2016 included)
%-------------------------------------------------------------------------------------------------------
% J. Leymarie, C. Hurlin. September 26, 2017.
% LEO, University of Orleans
%-------------------------------------------------------------------------------------------------------

clear ; clc ; close all ;  tic

help Main_Size_Conditional_MES

%==================%
%=== Parameters ===%
%==================%

cases=[250 250 ; 250 500 ; 500 250 ; 500 500; 2500 250; 2500 500];     % Sample sizes (first colum T, second coum N)

S=1000;                                            % Number of replications

alpha=0.05;                                        % alpha level for quantile of Y2

level=0.05;                                        % Nominal size of the tests

mmax=5;                                            % Maximum lag autocovariance for IND test

options=optimset('Display','off');                 % Options

%======================================================%
%=== Variance-Covariance and Correlation Parameters ===%
%======================================================%

a1=[0.149;0.065;0.075;0.855];                      % GARCH parameters for Y1 (GJR-DCC)

a2=[0.057;0.001;0.268;0.786];                      % GARCH parameters for Y2 (GJR-DCC)
                 
b=[0.663;0.041;0.754];                             % DCC parameters (rho_bar,a,b) (Normal DCC) 
                         
var1=a1(1)/(1-a1(2)-0.5*a1(3)-a1(4));              % Unconditional variance for Y1

var2=a2(1)/(1-a2(2)-0.5*a2(3)-a2(4));              % Unconditional variance for Y2

corr12=b(1);                                       % Unconditional correlation (Normal DCC)

cov12=corr12*sqrt(var1)*sqrt(var2);                % Unconditional covariance

Q_bar=[1 b(1);b(1) 1];                             % Unconditionnal correlation matrix 

theta0=[a1'  a2'  b'];                             % True value of the parameters

np=length(theta0);                                 % Number of estimated parameters

%======================%
%=== Initialization ===%
%======================%

ncases=size(cases,1);                              % Number of considered cases

uc=NaN(S,ncases); ind=NaN(S,mmax,ncases);          % UC and IND statistics without estimation risk

uc_risk=uc; ind_risk=ind;                          % UC and IND statistics with estimation risk

uc_c=uc; uc_c_kernel=uc;                           % UC robust statistics

ind_c=ind; ind_c_kernel=ind;                       % IND robust statistics

p_uc=uc; p_ind=ind;                                % UC and IND p_values without estimation risk

p_uc_risk=uc; p_ind_risk=ind;                      % UC and IND p_values with estimation risk

p_uc_c=uc;p_uc_c_kernel=uc;                        % UC robust p_values

p_ind_c=ind;p_ind_c_kernel=ind;                    % IND robust p_values 

for cas=1:ncases

    T=cases(cas,1);                                % In sample size

    N=cases(cas,2);                                % Out of sample size

    TT=T+N;                                        % Total size
    
%============================
%=== Simulations under H0 ===
%============================

parfor s=1:S                                       % Loop for the simulations
    
% if floor(s/100)==(s/100) ; disp(s); end          % Display the simulation number

converge=0;                                        % Convergence status (by default non convergence)
        
while converge==0                                  % Test on the convergence of the GJR-DCC estimation

Y=NaN(TT+500,2);Y(1,:)=0;                          % Initialization of Y1,Y2 series 

Sigma_t=NaN(2,2,TT+500);                           % Initialization of the cond. variance matrices

rho=NaN(TT+500,1);rho(1)=corr12;                   % Initialization of cond. correlations

S2=NaN(TT+500,2);S2(1,1)=var1;S2(1,2)=var2;        % Initialization of the cond. variances

Qlag=Q_bar;                                        % Lagged cond. correlation matrix

z=mvnrnd(zeros(2,1),eye(2),TT+500);                % Simulated innovations                                                                              
                                                            
for t=2:TT+500                                                  % Loop for time t
    
    S2(t,1)=a1(1)+a1(2)*Y(t-1,1)^2+...                          % Conditionnal variance of Y1 at time t
        a1(3)*(Y(t-1,1)<0)*Y(t-1,1).^2+...
        a1(4)*S2(t-1,1); 
    
    S2(t,2)=a2(1)+a2(2)*Y(t-1,2)^2+...                          % Conditionnal variance of Y2 at time t
        a2(3)*(Y(t-1,2)<0)*Y(t-1,2)^2+...
        a2(4)*S2(t-1,2);  
    
    eps=Y(t-1,:)./sqrt(S2(t-1,:));                              % Standardized Y1 Y2 at time t-1
    
    Q=(1-b(2)-b(3))*Q_bar+b(2)*(eps'*eps)+b(3)*Qlag;            % Q matrix at time t
    
    Qlag=Q;                                                     % Qlag becomes Q
    
    R=diag(diag(1./sqrt(Q)))*Q*diag(diag(1./sqrt(Q)));          % R matrix at time t
    
    rho(t)=R(1,2);                                              % Extraction of conditionnal correlation between Y1 Y2
   
    Sigma_t(:,:,t)=sqrt(diag(S2(t,:)))*R*sqrt(diag(S2(t,:)));   % Conditional covariance matrix at time t

    Y(t,:)=z(t,:)*chol(Sigma_t(:,:,t));                         % Simulated series

end                                                             % Loop on t
   
S2=S2(501:end,:);                                               % Removing 500 first values of S2

rho=rho(501:end,:);                                             % Removing 500 first values of rho
    
Sigma_t=Sigma_t(:,:,501:end);                                   % Removing 500 first values of cond. variance matrices
    
Y=Y(501:end,:);                                                 % Removing 500 first values of Y

VaR2=sqrt(S2(:,2))*norminv(alpha);                              % Y2's Value-at-Risk of level alpha

%===============%
%=== Figures ===%
%===============%

% plot([1:TT],Y(:,1),[1:TT],Y(:,2)),legend('Y1','Y2')
% plot([1:TT],Y(:,2),[1:TT],VaR2(:,:)),legend('Y2 return','Value at Risk at 5%'), xlabel('Time')
% plot([1:TT],S2(:,1),[1:TT],S2(:,2)),xlabel('Time'),legend('Variance cond. Y1','Variance cond. Y2') 
% plot(rho),legend('Correlation between Y1 Y2')

%============================
%=== Estimation In-Sample ===
%============================

[theta_in,loglik,Hcond,V_asy,scores,diagnostic] = dcc(Y(1:T,:),[],1,0,1,1,1,1,[],[],[],theta0,options); 

% disp('   theta0   theta_in')
% disp([theta0' theta_in'])
 
if (max(abs(mean(scores)))>0.001)||(min(abs(theta_in))<0.0001)||(sum(sum(isfinite(V_asy)))<np^2)  % Test for convergence (based on the scores)
            
    converge=0 ;                              % Not convergence
            
else                                          % Estimation has converged
                
converge=1 ;                                  % Convergence
             
end                                           % End condition if
            
end                                           % Loop while

garch_hat=[theta_in(1:4)' theta_in(5:8)'];    % Estimated GJR-GARCH parameters

dcc_hat=theta_in(10:11);                      % Estimated DCC parameters (without rho_bar)

Q_bar_hat=[1 theta_in(9);theta_in(9) 1];      % Estimated uncondional correlation

%==============================================================================%
%==== Evaluation of the correlations-variances in-sample and out-of-sample ====%
%==============================================================================%

[Sigma_hat,rho_hat,]= gjrdcc_evaluate(Y,garch_hat,dcc_hat,Q_bar_hat,T);% Evaluation over the full sample (initial condition based on in-sample period)

S2_hat=NaN(TT,2);                                                      % Estimated cond. variances (in-sample and out-of-sample)

S2_hat(:,1)=Sigma_hat(1,1,:);S2_hat(:,2)=Sigma_hat(2,2,:);             % Estimated cond. variances (in-sample and out-of-sample)

var1_in=S2_hat(1:T,1);var2_in=S2_hat(1:T,2);                           % Conditional variances (in-sample)

var1_out=S2_hat(T+1:end,1);var2_out=S2_hat(T+1:end,2);                 % Conditional variances (out-of-sample)

rho_in=rho_hat(1:T);rho_out=rho_hat(T+1:end);                          % Cond. correlations (in-sample and out-of-sample)

VaR2_in=sqrt(var2_in)*norminv(alpha);                                  % Value-at-Risk of level alpha for Y2 (in-sample)

VaR2_out=sqrt(var2_out)*norminv(alpha);                                % Value-at-Risk of level alpha for Y2 (in-sample)

Y1_out=Y(T+1:end,1);                                                   % OOS simulated realisations for Y1

Y2_out=Y(T+1:end,2);                                                   % OOS simulated realisations for Y2

Y_out=[Y1_out Y2_out];                                                 % OOS simulated realisations for [Y1 Y2]          

Sigma_out=Sigma_hat(:,:,T+1:end);                                      % OOS cond. variance matrices
    
%=======================================================================================%
%=== Derivatives of the cond. variance matrices with respect to estimated parameters ===%
%=======================================================================================%

z=10.^-8;                                                                              % Incremental value for derivatives

dSigma=NaN(np,3,TT);                                                                   % Initialization matrix of derivatives

for i=1:np                                                                             % Loop on the derivatives

    theta_in_z=theta_in;                                                               % Initialization 

    theta_in_z(i)=theta_in_z(i)+z;                                                     % Augmented value

    garch_hat_z=[theta_in_z(1:4)' theta_in_z(5:8)'];                                   % GARCH parameters

    dcc_hat_z=theta_in_z(10:11);                                                       % DCC parameters (without rho_bar)

    Q_bar_hat_z=[1 theta_in_z(9);theta_in_z(9) 1];                                     % Uncondional correlation

    [Sigma_hat_z,rho_hat_z,]= gjrdcc_evaluate(Y,garch_hat_z,dcc_hat_z,Q_bar_hat_z,T);  % Evaluation over the full sample (initial condition based on in-sample period)

    dSigma(i,1,:)=(reshape(Sigma_hat_z(1,1,:),TT,1)-reshape(Sigma_hat(1,1,:),TT,1))/z; % Derivatives of cond. variance var1

    dSigma(i,2,:)=(rho_hat_z-rho_hat)/z;                                               % Derivatives of cond. correlation

    dSigma(i,3,:)=(reshape(Sigma_hat_z(2,2,:),TT,1)-reshape(Sigma_hat(2,2,:),TT,1))/z; % Derivatives of cond. variance var2

end                                                                                                                                                  % End loop on the derivatives
        
dSigma_out=dSigma(:,:,T+1:end);                                                        % Out-of-sample derivatives
        
%=====================================================%
%=== Computation of H(alpha) series and RMES terms ===%
%=====================================================%

H=zeros(N,1);                                                                          % Ht(alpha) series (without estimation risk)

H_risk=zeros(N,1);                                                                     % Ht(alpha) series (with estimation risk)

RMESt_kernel=NaN(np,N);                                                                % RMES terms (kernel correction): derivatives with respect to estimated parameters

% RMESt=NaN(np,N);                                                                       % RMES terms: derivatives with respect to estimated parameters
        
for i=1:N                                                                              % Loop on out-of-sample periods
            
    if Y2_out(i)<=VaR2(T+i)                                                            % Without estimation risk
                
        H(i)=1-mvncdf([Y1_out(i) VaR2(T+i)],zeros(1,2),Sigma_t(:,:,T+i))/alpha;        % H(t)=1-u12 (without estimation risk)
                
    end                                                                                % End of if condition
            
    if Y2_out(i)<=VaR2_out(i)                                                          % With estimation risk
                
        H_risk(i)=1-mvncdf([Y1_out(i) VaR2_out(i)],zeros(1,2),Sigma_out(:,:,i))/alpha; % H(t)=1-u12 (with estimation risk)
                
    end                                                                                % End of if condition
            
  RMESt_kernel(:,i)=dSigma_out(:,:,i)*H_normal_derivatives_kernel(Sigma_out(:,:,i),Y_out(i,:),alpha,1/N);  % Correction term RMES (H kernel numerical derivative)               

%   RMESt(:,i)=dSigma_out(:,:,i)*[RMES_CoVaR_term(var1_out(i),var2_out(i),rho_out(i),VaR2_out(i),1,alpha);...% Correction term RMES (CoVaR methodology)
%        RMES_CoVaR_term(var1_out(i),var2_out(i),rho_out(i),VaR2_out(i),2,alpha);...
%        RMES_VaR2_term(var1_out(i),var2_out(i),rho_out(i),VaR2_out(i),3,alpha)];

end                                                                                                                                                                               % End of loop on i
        
%========================================%
%=== UC_MES - Unconditionnal backtest ===%  
%========================================%

% Without Estimation Risk

uc(s,cas)=sqrt(N)*(mean(H)-alpha/2)/sqrt(alpha*(1/3-alpha/4));            % Statistic UC without estimation risk

p_uc(s,cas)=2*normcdf(-abs(uc(s,cas)));                                   % UC p_value

% With Estimation Risk

uc_risk(s,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4));  % Statistic UC with estimation risk

p_uc_risk(s,cas)=2*normcdf(-abs(uc_risk(s,cas)));                         % UC p_value

% Robust Test Statistics

RMES_kernel=mean(RMESt_kernel,2);                                         % RMES mean (Kernel methodology)

% RMES=mean(RMESt,2);                                                       % RMES mean (CoVaR methodology)

uc_c_kernel(s,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)+N/T*RMES_kernel'*T*V_asy*RMES_kernel);   % Statistic UC robust (Kernel methodology)

p_uc_c_kernel(s,cas)=2*normcdf(-abs(uc_c_kernel(s,cas)));                 % UC p_value

% uc_c(s,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)+N/T*RMES'*T*V_asy*RMES); % Statistic UC robust (CoVaR methodology)

% p_uc_c(s,cas)=2*normcdf(-abs(uc_c(s,cas)));                               % UC p_value

%========================================%
%=== IND_MES - Conditionnal backtest  ===%
%========================================%
       
gammaHt_hat=NaN(mmax,1);gammaHt_hat_risk=gammaHt_hat;                     % Autocovariances of Ht(alpha)

rhoHt_hat=NaN(mmax,1);rhoHt_hat_risk=rhoHt_hat;                           % Autocorrelations of Ht(alpha)

R_kernel=NaN(mmax,np);                                                    % Correction term R (Ht kernel numerical derivative)  

% R=NaN(mmax,np);                                                           % Correction term R (CoVaR methodology)                            
          
for m=1:mmax                                                              % Loop on the lag

    % Autocorrelations with and without estimation risk

    Hlag=[NaN(m,1) ; H(1:N-m)];                                          % Lagged values

    gammaHt_hat(m)=(1/(N-m))*nansum((H-alpha/2).*(Hlag-alpha/2));        % Autocovariances of H(t)

    rhoHt_hat(m)=gammaHt_hat(m)/var(H);                                  % Autocorrelations of H(t)

    Hlag_risk=[NaN(m,1) ; H_risk(1:N-m)];                                % Lagged values with Estimation Risk

    gammaHt_hat_risk(m)=(1/(N-m))*nansum((H_risk-alpha/2).*(Hlag_risk-alpha/2));   % Autocovariances of H(t) with Estimation Risk

    rhoHt_hat_risk(m)=gammaHt_hat_risk(m)/var(H_risk);                             % Autocorrelations of H(t) with Estimation Risk

    % Matrix Ri and Rj 

    R_kernel(m,:)=(1/(alpha*(1/3-alpha/4)))*(nansum(repmat((Hlag_risk-alpha/2),1,np).*RMESt_kernel')/(N-m));   % Be careful: RMES must be a vector in the conditional case, do not use repmat

%     R(m,:)=(1/(alpha*(1/3-alpha/4)))*(nansum(repmat((Hlag_risk-alpha/2),1,np).*RMESt')/(N-m));                 % Be careful: RMES must be a vector in the conditional case, do not use repmat

    % Without Estimation Risk

    ind(s,m,cas)=N*sum(rhoHt_hat(1:m).^2);                           % Statistic IND without estimation risk (Box-Pierce)

    p_ind(s,m,cas)=1-chi2cdf(ind(s,m,cas),m);                            % IND p-value

    % With estimation risk

    ind_risk(s,m,cas)=N*sum(rhoHt_hat_risk(1:m).^2);                 % Statistic IND with estimation risk 

    p_ind_risk(s,m,cas)=1-chi2cdf(ind_risk(s,m,cas),m);                  % IND p-value

    % Robust test statistics

%     delta=eye(m)+N*R(1:m,:)*V_asy*R(1:m,:)';                             % Variance-covariance matrix of the corrected statistic

    delta_kernel=eye(m)+N*R_kernel(1:m,:)*V_asy*R_kernel(1:m,:)';        % Variance-covariance matrix of the corrected statistic (kernel)

    ind_c_kernel(s,m,cas)=N*rhoHt_hat_risk(1:m)'*pinv(delta_kernel)*rhoHt_hat_risk(1:m);  % Statistic IND robust (Ht kernel numerical derivative)

    p_ind_c_kernel(s,m,cas)=1-chi2cdf(ind_c_kernel(s,m,cas),m);                  % IND p-value

%     ind_c(s,m,cas)=N*rhoHt_hat_risk(1:m)'*pinv(delta)*rhoHt_hat_risk(1:m);   % Statistic IND robust (CoVaR methodology)
% 
%     p_ind_c(s,m,cas)=1-chi2cdf(ind_c(s,m,cas),m);                                % IND p-value

end                                                    % End loop on lags

end                                                    % Loop on S

disp('==================================================================================='),disp(' ')

fprintf('  In-sample size = %1.0f    Out-of-sample size = %1.0f   Number of replications = %1.0f \n',T,N,S),disp(' ')

fprintf('  Coverage rate for VaR = %1.3f      Size = %1.3f \n',alpha,level),disp(' ')

fprintf('  Max lag (for IND test) = %1.0f  \n',mmax)

disp(' '),disp('  ==== Test UC ====')

disp(' '),fprintf('  Empirical size = %1.4f  \n',mean(p_uc(:,cas)<=level))   

disp(' '),fprintf('  Empirical size (with ER) = %1.4f \n',mean(p_uc_risk(:,cas)<=level))    

disp(' '),fprintf('  Empirical size (robust, Kernel method) = %1.4f \n',mean(p_uc_c_kernel(:,cas)<=level))

% disp(' '),fprintf('  Empirical size (robust, CoVaR method) = %1.4f \n',mean(p_uc_c(:,cas)<=level))  

disp(' '), disp('  ==== Test IND ====')

disp(' '),disp('  Lags ')  

disp(sprintf('   %1.4f ',(1:1:mmax)) )

disp(' ');disp('  Empirical size ')

disp(sprintf('   %1.4f ',mean(p_ind(:,1:mmax,cas)<=level)))

disp(' ');disp('  Empirical size (with ER)')

disp(sprintf('   %1.4f ',mean(p_ind_risk(:,1:mmax,cas)<=level))) 

disp(' ');disp('  Empirical size (robust, Kernel method)')

disp(sprintf('   %1.4f ',mean(p_ind_c_kernel(:,1:mmax,cas)<=level))) 

% disp(' ');disp('  Empirical size (robust, CoVaR method)')
% 
% disp(sprintf('   %1.4f ',mean(p_ind_c(:,1:mmax,cas)<=level))) 

disp(' ')

end

toc

% uc_H0=uc;uc_risk_H0=uc_risk;uc_c_kernel_H0=uc_c_kernel;
% 
% ind_H0=ind;ind_risk_H0=ind_risk;ind_c_kernel_H0=ind_c_kernel;
% 
% save Size_Conditional_MES_forpowersizecorrected uc_H0 uc_risk_H0 uc_c_kernel_H0 ind_H0 ind_risk_H0 ind_c_kernel_H0
