%----------------------------------------------------------------------------------------------------------
% PURPOSE: "Backtesting Marginal Expected Shortfall and Related Systemic Risk Measures"
%           Banulescu, Leymarie, Hurlin and Scaillet (2016)
%-----------------------------------------------------------------------------------------------------------
% Backtesting test for the daily Delta CoVaR :
% Empirical size got from Monte Carlo simulations based on a Gaussian static model with estimation risk 
%-----------------------------------------------------------------------------------------------------------
%   H1 : Empirical Power for 3 configuations (H1A, H1B, H1C)
%-----------------------------------------------------------------------------------------------------------
% December, 28, 2016
% J. Leymarie, University of Orleans
%-----------------------------------------------------------------------------------------------------------
% Note : Parameters are calibrated based on the estimation of GJRDCC parameters of the daily log returns 
%        of BAC and CRSP market weighted index (2012 - 2016)
%-----------------------------------------------------------------------------------------------------------

clear, clc, close all,

help Main_Mower_Marginal_DeltaCoVaR

load Size_Marginal_DeltaCoVaR_forpowersizecorrected

%==================%
%==  Parameters  ==%
%==================%

cases=[250 250 ; 250 500 ; 500 250 ; 500 500; 2500 250; 2500 500]; % Sample sizes (first colum T, second coum N)

S=10000;                                                           % Number of replications

alpha=0.05;                                                        % One-sided Y2 truncation and CoVaR level

btilde=[0.25;0.75];binf=btilde(1);bsup=btilde(2);                  % Two-sided Y2 truncation

a1=[0.149;0.065;0.075;0.855];                                      % GARCH parameters for the firm

a2=[0.057;0.001;0.268;0.786];                                      % GARCH paramters for the market
                         
var1=a1(1)/(1-a1(2)-0.5*a1(3)-a1(4));                              % Unconditional variance for Y1

var2=a2(1)/(1-a2(2)-0.5*a2(3)-a2(4));                              % Unconditional variance for Y2

corr12=0.663;                                                      % Unconditional correlation

cov12=corr12*sqrt(var1)*sqrt(var2);                                % Unconditional covariance

Sigma=[var1 cov12 ; cov12 var2];                                   % Unconditional variance-covariance matrix

level=0.05;                                                        % Nominal size of the test

options=optimset;                                                  % Options for parameter's estimation

covar_options = optimset('TolX',10^-20);                           % Covar termination tolerance set to 10^-20 for accuracy

%=====================
%=== Configurations H1 ===
%=====================

factor=[0.75 0.5 0.25 ;...                        % Decreasing Factors for H1 : first line: var1, second line :var2, third line: correlation
        0.75 0.5 0.25 ;...
        0.75 0.5 0.25]; 

% factor=[0.9 0.8 0.7 ;...                        % Decreasing Factors for H1 : first line: var1, second line :var2, third line: correlation
%         0.9 0.8 0.7 ;...
%         0.9 0.8 0.7]; 

nfactor=numel(factor);                            % Number of H1 hypothesis considered

%======================%
%=== Initialization ===%
%======================%

ncases=size(cases,1);                                              % Number of considered cases

uc_dcovar=NaN(S,nfactor,ncases); puc_dcovar=uc_dcovar;             % Initialization of uc dcovar without estimation risk

uc_dcovar_risk=uc_dcovar;                                          % Initialization of uc dcovar with estimation risk

uc_dcovar_c=uc_dcovar;                                             % Initialization of uc dcovar derivatives of expectations corrected

uc_dcovar_ck=uc_dcovar;                                            % Initialization of uc dcovar kernel corrected

uc_h1=uc_dcovar; uc_h2=uc_dcovar; uc_h1_risk=uc_dcovar;

uc_h2_risk=uc_dcovar; uc_h1_ck=uc_h1; uc_h2_ck=uc_h1;

for cas=1:ncases

    T=cases(cas,1);                                 % In sample size

    N=cases(cas,2);                                 % Out of sample size

    TT=T+N;                                         % Total size

    %============================
    %=== Simulations under H0 ===
    %============================
    
for h_conf=1:1:numel(factor)                                          % Loop on the configuations H1A, H1B, H1C             
       
parfor s=1:S                                                          % Loop for the number of replications

    Y=mvnrnd(zeros(2,1),Sigma,T+N);                                   % Y sample

    Y_in=Y(1:T,:);                                                    % Y_in sample

    Y_out=Y(T+1:T+N,:);                                               % Y_out sample
    
    Sigma_in_H0=cov(Y(1:T,:),1);                                      % Estimated unconditional variance-covariance matrix 

    theta_in_H0=[Sigma_in_H0(1,1) ...                                 % Estimated parameters in-sample
              Sigma_in_H0(1,2)/sqrt(Sigma_in_H0(2,2)*Sigma_in_H0(1,1))...    
              Sigma_in_H0(2,2)]'; 
          
    if h_conf<=size(factor,2)                                         % H1A: error in variance1 estimate

        var1_H1=Sigma(1,1)*factor(1,h_conf);                          % H1A: error in variance1 estimate

        theta_in=[var1_H1 ; theta_in_H0(2:end)];                      % Constrained estimates for H1A

        Mask=[0 1 0 ; 0 0 1];                                         % Mask matrix allows to delete the first row of MES (variance1)

    elseif (h_conf>size(factor,2))&&(h_conf<=2*size(factor,2))        % H1B: error in variance2 estimate

        var2_H1=Sigma(2,2)*factor(2,h_conf-size(factor,2));           % H1B: error in variance2 estimate

        theta_in=[theta_in_H0(1:2) ; var2_H1 ];                       % Constrained estimates for H1B

        Mask=[1 0 0 ; 0 1 0];                                         % Mask matrix allows to delete the third row of MES (variance2)

    elseif h_conf>2*size(factor,2)                                    % H1C: error in correlation estimate

        rho_H1=Sigma(1,2)/sqrt(prod(diag(Sigma)))*factor(3,h_conf-2*size(factor,2));   % H1C: error in correlation estimate                

        theta_in=[theta_in_H0(1) ; rho_H1; theta_in_H0(3)];           % Constrained estimates for H1C

        Mask=[1 0 0 ; 0 0 1];                                         % Mask matrix allows to delete the second row of MES (correlation)                

    end   
    
    Sigma_in=[theta_in(1)  theta_in(2)*sqrt(theta_in(1)*theta_in(3)) ;...
              theta_in(2)*sqrt(theta_in(1)*theta_in(3)) theta_in(3)];

    var1_in=Sigma_in(1,1);                                            % Estimated unconditional variance for Y1

    var2_in=Sigma_in(2,2);                                            % Estimated unconditional variance for Y1

    rho_in=Sigma_in(1,2)/sqrt(var2_in*var1_in);                       % In sample correlation

    cov_in=Sigma_in(1,2);                                             % In sample covariance

    Hessian=hessian_2sided('loglikelihood_normal',...                 % Hessian matrix under the null (Loglik is the opposite of the likelihood)
            theta_in_H0,Y(1:T,:));                                    % Estimation error made in H1 is supposed missed by the estimation procedure   

    V_asy=Mask*pinv(Hessian)*Mask';                                   % Asymptotic variance-covariance of ML estimators

    %====================================%
    %== Delta CoVaR(alpha) computation ==%
    %====================================%
    
    %-----------------------------%
    %-- Without Estimation Risk --%
    %-----------------------------%

    VaR2alpha=norminv(alpha,0,sqrt(Sigma(2,2)));                    % Y2's VaR of order alpha
    
    VaR2binf=norminv(binf,0,sqrt(Sigma(2,2)));                      % Y2's VaR of order binf         
    
    VaR2bsup=norminv(bsup,0,sqrt(Sigma(2,2)));                      % Y2's VaR of order bsup

    obj = @(x) (mvncdf([x;VaR2alpha],zeros(2,1),Sigma)/normcdf(VaR2alpha,0,sqrt(Sigma(2,2)))-alpha)^2;

    x0=0; covar1=fminsearch(obj,x0,covar_options);                  % CoVaR(alpha,alpha) 
    
    obj = @(x) ((mvncdf([x;VaR2bsup],zeros(2,1),Sigma)-mvncdf([x;VaR2binf],zeros(2,1),Sigma))/...
        (normcdf(VaR2bsup,0,sqrt(Sigma(2,2)))-normcdf(VaR2binf,0,sqrt(Sigma(2,2))))-alpha)^2;

    x0=0; covar2=fminsearch(obj,x0,covar_options);                  % "Median" CoVaR(alpha,btilde)

    dcovar=covar1-covar2;                                           % Delta CoVaR
    
    %-----------------------------%
    %--   With Estimation Risk  --%
    %-----------------------------%

    VaR2alpha_in=norminv(alpha,0,sqrt(Sigma_in(2,2)));              % ER Y2's VaR of order alpha
    
    VaR2binf_in=norminv(binf,0,sqrt(Sigma_in(2,2)));                % ER Y2's VaR of order binf            
    
    VaR2bsup_in=norminv(bsup,0,sqrt(Sigma_in(2,2)));                % ER Y2's VaR of order bsup

    obj = @(x) (mvncdf([x;VaR2alpha_in],zeros(2,1),Sigma_in)/normcdf(VaR2alpha_in,0,sqrt(Sigma_in(2,2)))-alpha)^2;

    x0=0; covar1_in=fminsearch(obj,x0,covar_options);               % ER CoVaR(alpha,alpha)

    obj = @(x) ((mvncdf([x;VaR2bsup_in],zeros(2,1),Sigma_in)-mvncdf([x;VaR2binf_in],zeros(2,1),Sigma_in))/...
        (normcdf(VaR2bsup_in,0,sqrt(Sigma_in(2,2)))-normcdf(VaR2binf_in,0,sqrt(Sigma_in(2,2))))-alpha)^2;

    x0=0; covar2_in=fminsearch(obj,x0,covar_options);               % ER "Median" CoVaR(alpha,btilde)

    dcovar_in=covar1_in-covar2_in;                                  % ER Delta CoVaR
    
    % disp([covar1_in chou_owen_covar(var1_in,rho_in,alpha,alpha)]) % covar1_in compared to Owen & Chou approximation
    
    %========================%
    %==  Backtesting Test  ==%
    %========================%
    
    %-----------------------------%
    %-- Without Estimation Risk --%
    %-----------------------------%
    
    ht=[(Y_out(:,1)<covar1).*(Y_out(:,2)<VaR2alpha), ...                      % [h1t,h2t] violations (without ER)
        (Y_out(:,1)<covar2).*(Y_out(:,2)>VaR2binf)&(Y_out(:,2)<VaR2bsup)];

    htbar=mean(ht)';                                                          % h bar violation (without ER)

    muh=[alpha^2;alpha*(bsup-binf)];                                          % h bar expectation under H0

    sigmah=[alpha^2*(1-alpha^2) -alpha^3*(bsup-binf);...                      % h bar variance under H0
        -alpha^3*(bsup-binf) alpha*(bsup-binf)*(1-alpha*(bsup-binf))];

    uc_dcovar(s,h_conf,cas)=N*(htbar-muh)'*pinv(sigmah)*(htbar-muh);           % uc_dcovar statistic (without ER)
    
    uc_h1(s,h_conf,cas)=sqrt(N)*(htbar(1)-muh(1))/sqrt(sigmah(1,1));           % uc for h1 violation (without ER)

    uc_h2(s,h_conf,cas)=sqrt(N)*(htbar(2)-muh(2))/sqrt(sigmah(2,2));           % uc for h2 violation (without ER)

    %-----------------------------%
    %--   With Estimation Risk  --%
    %-----------------------------%

    ht_risk=[(Y_out(:,1)<covar1_in).*(Y_out(:,2)<VaR2alpha_in), ...           % [h1t,h2t] violations (with ER)
        (Y_out(:,1)<covar2_in).*(Y_out(:,2)>VaR2binf_in)&(Y_out(:,2)<VaR2bsup_in)];
    
    htbar_risk=mean(ht_risk)';                                                % h bar violation (with ER)

    uc_dcovar_risk(s,h_conf,cas)=N*(htbar_risk-muh)'*pinv(sigmah)*(htbar_risk-muh);  % uc_dcovar_risk statistic (with ER)
    
    uc_h1_risk(s,h_conf,cas)=sqrt(N)*(htbar_risk(1)-muh(1))/sqrt(sigmah(1,1));       % uc for h1 violation (with ER)

    uc_h2_risk(s,h_conf,cas)=sqrt(N)*(htbar_risk(2)-muh(2))/sqrt(sigmah(2,2));       % uc for h2 violation (with ER)

    %---------------------------------------------------------------------%
    %--       ER corrected by deriving the [h1,h2] expectations         --%
    %---------------------------------------------------------------------%
    
    z=10^-8;                                                            % Incremental value for derivatives
    
%     [dEh1 , dEh2] =...                                                  % E[h1t], E[h2t] derivatives with respect to [var1;rho;var2]
%          h_normal_derivatives_expectation_deltacovar...
%         (Sigma_in,covar1_in,covar2_in,covar_options,z,alpha,btilde);
%     
%     r1covar=Mask*dEh1;                                                  % r1covar term is the same than dEh1 in static case
%     
%     r2covar=Mask*dEh2;                                                  % r2covar term is the same than dEh2 in static case
%     
%     sigma_c=sigmah+...                                                  % h bar corrected variance from ER
%         [N*r1covar'*V_asy*r1covar N*r1covar'*V_asy*r2covar ; ...
%          N*r1covar'*V_asy*r2covar N*r2covar'*V_asy*r2covar];
%     
%     uc_dcovar_c(s,h_conf,cas)=N*(htbar_risk-muh)'*pinv(sigma_c)*(htbar_risk-muh);  % uc_dcovar_c statistic(corrected from ER)
    
    %---------------------------------------------------------------------%
    %-- ER corrected by deriving the smoothed [h1,h2] (kernel smoother) --%
    %---------------------------------------------------------------------%

    [dh1 , dh2] =...                                                    % Partial derivatives with respect to [var1;rho;var2]
        h_normal_derivatives_kernel_deltacovar...
        (Sigma_in,Y_out,z,alpha,btilde,1/N);
    
    r1covar_k=Mask*mean(dh1,2);                                         % kernel r1covar term
    
    r2covar_k=Mask*mean(dh2,2);                                         % kernel r2covar term
    
    sigma_ck=sigmah+...                                                 % h bar corrected variance from ER
        [N*r1covar_k'*V_asy*r1covar_k N*r1covar_k'*V_asy*r2covar_k ; ...
         N*r1covar_k'*V_asy*r2covar_k N*r2covar_k'*V_asy*r2covar_k];
  
    uc_dcovar_ck(s,h_conf,cas)=N*(htbar_risk-muh)'*pinv(sigma_ck)*(htbar_risk-muh);% uc_dcovar_ck statistic (corrected from ER)
    
    uc_h1_ck(s,h_conf,cas)=sqrt(N)*(htbar_risk(1)-muh(1))...            % uc for h1 violation (with ER)
        /sqrt(sigmah(1,1)+N*r1covar_k'*V_asy*r1covar_k);      

    uc_h2_ck(s,h_conf,cas)=sqrt(N)*(htbar_risk(2)-muh(2))/...           % uc for h2 violation (with ER)
        sqrt(sigmah(2,2)+N*r2covar_k'*V_asy*r2covar_k);      
    

end                                                    % Loop on S

disp('==================================================================================='),disp(' ')

if h_conf<=size(factor,2)                                             % H1A: error in variance1 estimate

    fprintf('  Hypothesis H1A: decrease in variance(Y1) by %1.4f \n',1-factor(1,h_conf)),disp(' ')

elseif (h_conf>size(factor,2))&&(h_conf<=2*size(factor,2))            % H1B: error in variance2 estimate

    fprintf('  Hypothesis H1B: decrease in variance(Y2) by %1.4f \n',1-factor(2,h_conf-size(factor,2))),disp(' ')            

elseif h_conf>2*size(factor,2)                                        % H1C: error in correlation estimate

    fprintf('  Hypothesis H1C: decrease in correlation by %1.4f \n',1-factor(3,h_conf-2*size(factor,2))),disp(' ')                

end                                                                   % End of loop on h_conf 

fprintf('  In-sample size = %1.0f    Out-of-sample size = %1.0f   Number of replications = %1.0f \n',T,N,S),disp(' ')

fprintf('  alpha = %1.3f     binf = %1.3f     bsup = %1.3f     Size = %1.3f     \n',alpha,binf,bsup,level),disp(' ')

fprintf('  == Delta Covar test =='), disp(' ')

disp(' '),fprintf('  Empirical size = %1.4f \n',mean(abs(uc_dcovar(:,h_conf,cas))>quantile(abs(uc_H0(:,cas)),1-level)))  

disp(' '),fprintf('  Empirical size (With ER) = %1.4f \n',mean(abs(uc_dcovar_risk(:,h_conf,cas))>quantile(abs(uc_risk_H0(:,cas)),1-level))) 

disp(' '),fprintf('  Empirical size (robust, deriving the kernel smoothed [h1,h2]) = %1.4f \n',mean(abs(uc_dcovar_ck(:,h_conf,cas))>quantile(abs(uc_c_kernel_H0(:,cas)),1-level))) 

disp(' '),fprintf('  == h1 violation test =='),disp(' ')

disp(' '),fprintf('  Empirical size h1 = %1.4f \n',mean(abs(uc_h1(:,h_conf,cas))>quantile(abs(uc_h1_H0(:,cas)),1-level)))

disp(' '),fprintf('  Empirical size h1 (With ER) = %1.4f \n',mean(abs(uc_h1_risk(:,h_conf,cas))>quantile(abs(uc_h1_risk_H0(:,cas)),1-level)))

disp(' '),fprintf('  Empirical size h1 (robust, deriving the kernel smoothed [h1,h2]) = %1.4f \n',mean(abs(uc_h1_ck(:,h_conf,cas))>quantile(abs(uc_h1_c_kernel_H0(:,cas)),1-level)))

disp(' '),fprintf('  == h2 violation test =='),disp(' ')

disp(' '),fprintf('  Empirical size h2 = %1.4f \n',mean(abs(uc_h2(:,h_conf,cas))>quantile(abs(uc_h2_H0(:,cas)),1-level)))

disp(' '),fprintf('  Empirical size h2 (With ER) = %1.4f \n',mean(abs(uc_h2_risk(:,h_conf,cas))>quantile(abs(uc_h2_risk_H0(:,cas)),1-level)))

disp(' '),fprintf('  Empirical size h2 (robust, deriving the kernel smoothed [h1,h2]) = %1.4f \n',mean(abs(uc_h2_ck(:,h_conf,cas))>quantile(abs(uc_h2_c_kernel_H0(:,cas)),1-level)))

end

end


