%-------------------------------------------------------------------------------------------------------
% PURPOSE: "Backtesting Marginal Expected Shortfall and Related Systemic Risk Measures"
%           Banulescu, Leymarie, Hurlin and Scaillet (2016)
%-------------------------------------------------------------------------------------------------------
% Backtesting test for the daily Delta CoVaR :
% Empirical size got from Monte Carlo simulations based on a Gaussian static model with estimation risk 
%-----------------------------------------------------------------------------------------------------------
% December, 1, 2016
% J. Leymarie, University of Orleans
%-----------------------------------------------------------------------------------------------------------
% Note : Parameters are calibrated based on the estimation of GJRDCC parameters of the daily log returns 
%        of BAC and CRSP market weighted index (2012 - 2016)
%-----------------------------------------------------------------------------------------------------------

clear, clc, close all,

help Main_Size_Marginal_DeltaCoVaR

%==================%
%==  Parameters  ==%
%==================%

cases=[250 250 ; 250 500 ; 500 250 ; 500 500; 2500 250; 2500 500]; % Sample sizes (first colum T, second coum N)

S=10000;                                                           % Number of replications

alpha=0.05;                                                        % One-sided Y2 truncation and CoVaR level

btilde=[0.25;0.75];binf=btilde(1);bsup=btilde(2);                  % Two-sided Y2 truncation

a1=[0.149;0.065;0.075;0.855];                                      % GARCH parameters for the firm

a2=[0.057;0.001;0.268;0.786];                                      % GARCH paramters for the market
                         
var1=a1(1)/(1-a1(2)-0.5*a1(3)-a1(4));                              % Unconditional variance for Y1

var2=a2(1)/(1-a2(2)-0.5*a2(3)-a2(4));                              % Unconditional variance for Y2

corr12=0.663;                                                      % Unconditional correlation

cov12=corr12*sqrt(var1)*sqrt(var2);                                % Unconditional covariance

sigma=[var1 cov12 ; cov12 var2];                                   % Unconditional variance-covariance matrix

level=0.05;                                                        % Nominal size of the test

options=optimset;                                                  % Options for parameter's estimation

covar_options = optimset('TolX',10^-20);                           % Covar termination tolerance set to 10^-20 for accuracy

%======================%
%=== Initialization ===%
%======================%

ncases=size(cases,1);                                              % Number of considered cases

uc_dcovar=NaN(S,ncases); puc_dcovar=uc_dcovar;                     % Initialization of uc dcovar without estimation risk

uc_dcovar_risk=uc_dcovar; puc_dcovar_risk=uc_dcovar;               % Initialization of uc dcovar with estimation risk

uc_dcovar_c=uc_dcovar; puc_dcovar_c=uc_dcovar;                     % Initialization of uc dcovar derivatives of expectations corrected

uc_dcovar_ck=uc_dcovar;puc_dcovar_ck=uc_dcovar;                    % Initialization of uc dcovar kernel corrected

uc_h1=uc_dcovar; uc_h2=uc_dcovar; uc_h1_risk=uc_dcovar;

uc_h2_risk=uc_dcovar; puc_h1=uc_dcovar; puc_h2=uc_dcovar; 

puc_h1_risk=uc_dcovar; puc_h2_risk=uc_dcovar;

uc_h1_ck=uc_h1; uc_h2_ck=uc_h1; puc_h1c=uc_h1; puc_h2c=uc_h1;

puc_h1_ck=uc_h1; puc_h2_ck=uc_h2;

for cas=1:ncases

    T=cases(cas,1);                                 % In sample size

    N=cases(cas,2);                                 % Out of sample size

    TT=T+N;                                         % Total size

    %============================
    %=== Simulations under H0 ===
    %============================

parfor s=1:S

    Y=mvnrnd(zeros(2,1),sigma,T+N);                                   % Y sample

    Y_in=Y(1:T,:);                                                    % Y_in sample

    Y_out=Y(T+1:T+N,:);                                               % Y_out sample
    
    sigma_in=cov(Y(1:T,:),1);                                         % Estimated unconditional variance-covariance matrix 

    var1_in=sigma_in(1,1);                                            % Estimated unconditional variance for Y1

    var2_in=sigma_in(2,2);                                            % Estimated unconditional variance for Y1

    rho_in=sigma_in(1,2)/sqrt(var2_in*var1_in);                       % In sample correlation

    cov_in=sigma_in(1,2);                                             % In sample covariance

    theta_in=[var1_in rho_in var2_in]';                               % Estimated parameters in-sample

    Hessian=hessian_2sided('loglikelihood_normal',theta_in,Y(1:T,:)); % Hessian matrix (Loglik is the opposite of the likelihood)

    V_asy=pinv(Hessian);                                              % Asymptotic variance-covariance of ML estimators

    %====================================%
    %== Delta CoVaR(alpha) computation ==%
    %====================================%
    
    %-----------------------------%
    %-- Without Estimation Risk --%
    %-----------------------------%

    VaR2alpha=norminv(alpha,0,sqrt(sigma(2,2)));                    % Y2's VaR of order alpha
    
    VaR2binf=norminv(binf,0,sqrt(sigma(2,2)));                      % Y2's VaR of order binf         
    
    VaR2bsup=norminv(bsup,0,sqrt(sigma(2,2)));                      % Y2's VaR of order bsup

    obj = @(x) (mvncdf([x;VaR2alpha],zeros(2,1),sigma)/normcdf(VaR2alpha,0,sqrt(sigma(2,2)))-alpha)^2;

    x0=0; covar1=fminsearch(obj,x0,covar_options);                  % CoVaR(alpha,alpha) 
    
    obj = @(x) ((mvncdf([x;VaR2bsup],zeros(2,1),sigma)-mvncdf([x;VaR2binf],zeros(2,1),sigma))/...
        (normcdf(VaR2bsup,0,sqrt(sigma(2,2)))-normcdf(VaR2binf,0,sqrt(sigma(2,2))))-alpha)^2;

    x0=0; covar2=fminsearch(obj,x0,covar_options);                  % "Median" CoVaR(alpha,btilde)

    dcovar=covar1-covar2;                                           % Delta CoVaR
    
    %-----------------------------%
    %--   With Estimation Risk  --%
    %-----------------------------%

    VaR2alpha_in=norminv(alpha,0,sqrt(sigma_in(2,2)));              % ER Y2's VaR of order alpha
    
    VaR2binf_in=norminv(binf,0,sqrt(sigma_in(2,2)));                % ER Y2's VaR of order binf            
    
    VaR2bsup_in=norminv(bsup,0,sqrt(sigma_in(2,2)));                % ER Y2's VaR of order bsup

    obj = @(x) (mvncdf([x;VaR2alpha_in],zeros(2,1),sigma_in)/normcdf(VaR2alpha_in,0,sqrt(sigma_in(2,2)))-alpha)^2;

    x0=0; covar1_in=fminsearch(obj,x0,covar_options);               % ER CoVaR(alpha,alpha)

    obj = @(x) ((mvncdf([x;VaR2bsup_in],zeros(2,1),sigma_in)-mvncdf([x;VaR2binf_in],zeros(2,1),sigma_in))/...
        (normcdf(VaR2bsup_in,0,sqrt(sigma_in(2,2)))-normcdf(VaR2binf_in,0,sqrt(sigma_in(2,2))))-alpha)^2;

    x0=0; covar2_in=fminsearch(obj,x0,covar_options);               % ER "Median" CoVaR(alpha,btilde)

    dcovar_in=covar1_in-covar2_in;                                  % ER Delta CoVaR
    
    % disp([covar1_in chou_owen_covar(var1_in,rho_in,alpha,alpha)]) % covar1_in compared to Owen & Chou approximation
    
    %========================%
    %==  Backtesting Test  ==%
    %========================%
    
    %-----------------------------%
    %-- Without Estimation Risk --%
    %-----------------------------%
    
    ht=[(Y_out(:,1)<covar1).*(Y_out(:,2)<VaR2alpha), ...                      % [h1t,h2t] violations (without ER)
        (Y_out(:,1)<covar2).*(Y_out(:,2)>VaR2binf)&(Y_out(:,2)<VaR2bsup)];

    htbar=mean(ht)';                                                          % h bar violation (without ER)

    muh=[alpha^2;alpha*(bsup-binf)];                                          % h bar expectation under H0

    sigmah=[alpha^2*(1-alpha^2) -alpha^3*(bsup-binf);...                      % h bar variance under H0
        -alpha^3*(bsup-binf) alpha*(bsup-binf)*(1-alpha*(bsup-binf))];

    uc_dcovar(s,cas)=N*(htbar-muh)'*pinv(sigmah)*(htbar-muh);                  % uc_dcovar statistic (without ER)
    
    puc_dcovar(s,cas)=1-chi2cdf(uc_dcovar(s,cas),2);                           % p-value of uc_dcovar (without ER)

    uc_h1(s,cas)=sqrt(N)*(htbar(1)-muh(1))/sqrt(sigmah(1,1));                  % uc for h1 violation (without ER)

    puc_h1(s,cas)=2*normcdf(-abs(uc_h1(s,cas)));                               % p-value of uc_h1 (without ER)

    uc_h2(s,cas)=sqrt(N)*(htbar(2)-muh(2))/sqrt(sigmah(2,2));                  % uc for h2 violation (without ER)

    puc_h2(s,cas)=2*normcdf(-abs(uc_h2(s,cas)));                               % p-value of uc_h2 (without ER)

    %-----------------------------%
    %--   With Estimation Risk  --%
    %-----------------------------%

    ht_risk=[(Y_out(:,1)<covar1_in).*(Y_out(:,2)<VaR2alpha_in), ...           % [h1t,h2t] violations (with ER)
        (Y_out(:,1)<covar2_in).*(Y_out(:,2)>VaR2binf_in)&(Y_out(:,2)<VaR2bsup_in)];
    
    htbar_risk=mean(ht_risk)';                                                % h bar violation (with ER)

    uc_dcovar_risk(s,cas)=N*(htbar_risk-muh)'*pinv(sigmah)*(htbar_risk-muh);  % uc_dcovar_risk statistic (with ER)
    
    puc_dcovar_risk(s,cas)=1-chi2cdf(uc_dcovar_risk(s,cas),2);                % p-value of uc_dcovar_risk (with ER)

    uc_h1_risk(s,cas)=sqrt(N)*(htbar_risk(1)-muh(1))/sqrt(sigmah(1,1));       % uc for h1 violation (with ER)

    puc_h1_risk(s,cas)=2*normcdf(-abs(uc_h1_risk(s,cas)));                    % p-value of uc_h1 (with ER)

    uc_h2_risk(s,cas)=sqrt(N)*(htbar_risk(2)-muh(2))/sqrt(sigmah(2,2));       % uc for h2 violation (with ER)

    puc_h2_risk(s,cas)=2*normcdf(-abs(uc_h2_risk(s,cas)));                    % p-value of uc_h2 (with ER)

    %---------------------------------------------------------------------%
    %--       ER corrected by deriving the [h1,h2] expectations         --%
    %---------------------------------------------------------------------%
    
    z=10^-8;                                                            % Incremental value for derivatives
    
    [dEh1 , dEh2] =...                                                  % E[h1t], E[h2t] derivatives with respect to [var1;rho;var2]
        h_normal_derivatives_expectation_deltacovar...
        (sigma_in,covar1_in,covar2_in,covar_options,z,alpha,btilde);
    
    r1covar=dEh1;                                                       % r1covar term is the same than dEh1 in static case
    
    r2covar=dEh2;                                                       % r2covar term is the same than dEh2 in static case
    
    sigma_c=sigmah+...                                                  % h bar corrected variance from ER
        [N*r1covar'*V_asy*r1covar N*r1covar'*V_asy*r2covar ; ...
         N*r1covar'*V_asy*r2covar N*r2covar'*V_asy*r2covar];
    
    uc_dcovar_c(s,cas)=N*(htbar_risk-muh)'*pinv(sigma_c)*(htbar_risk-muh);  % uc_dcovar_c statistic(corrected from ER)
    
    puc_dcovar_c(s,cas)=1-chi2cdf(uc_dcovar_c(s,cas),2);                % p-value of uc_dcovar_c

    %---------------------------------------------------------------------%
    %-- ER corrected by deriving the smoothed [h1,h2] (kernel smoother) --%
    %---------------------------------------------------------------------%

    [dh1 , dh2] =...                                                    % Partial derivatives with respect to [var1;rho;var2]
        h_normal_derivatives_kernel_deltacovar...
        (sigma_in,Y_out,z,alpha,btilde,1/N);
    
    r1covar_k=mean(dh1,2);                                              % kernel r1covar term
    
    r2covar_k=mean(dh2,2);                                              % kernel r2covar term
    
    sigma_ck=sigmah+...                                                 % h bar corrected variance from ER
        [N*r1covar_k'*V_asy*r1covar_k N*r1covar_k'*V_asy*r2covar_k ; ...
         N*r1covar_k'*V_asy*r2covar_k N*r2covar_k'*V_asy*r2covar_k];
  
    uc_dcovar_ck(s,cas)=N*(htbar_risk-muh)'*pinv(sigma_ck)*(htbar_risk-muh);% uc_dcovar_ck statistic (corrected from ER)
    
    puc_dcovar_ck(s,cas)=1-chi2cdf(uc_dcovar_ck(s,cas),2);              % p-value of uc_dcovar_ck

    uc_h1_ck(s,cas)=sqrt(N)*(htbar_risk(1)-muh(1))...                   % uc for h1 violation (with ER)
        /sqrt(sigmah(1,1)+N*r1covar_k'*V_asy*r1covar_k);      

    puc_h1_ck(s,cas)=2*normcdf(-abs(uc_h1_ck(s,cas)));                  % p-value of uc_h1 (with ER)

    uc_h2_ck(s,cas)=sqrt(N)*(htbar_risk(2)-muh(2))/...                  % uc for h2 violation (with ER)
        sqrt(sigmah(2,2)+N*r2covar_k'*V_asy*r2covar_k);      

    puc_h2_ck(s,cas)=2*normcdf(-abs(uc_h2_ck(s,cas)));                  % p-value of uc_h2 (with ER)
    

end                                                         % Loop on S

disp('==================================================================================='),disp(' ')

fprintf('  In-sample size = %1.0f    Out-of-sample size = %1.0f   Number of replications = %1.0f \n',T,N,S),disp(' ')

fprintf('  alpha = %1.3f     binf = %1.3f     bsup = %1.3f     Size = %1.3f     \n',alpha,binf,bsup,level),disp(' ')

disp(' '),fprintf('  == Delta Covar test =='), disp(' ')

disp(' '),fprintf('  Empirical size = %1.4f \n',mean(puc_dcovar(:,cas)<=level))

disp(' '),fprintf('  Empirical size (With ER) = %1.4f \n',mean(puc_dcovar_risk(:,cas)<=level))

disp(' '),fprintf('  Empirical size (robust, deriving the [h1,h2] expectations) = %1.4f \n',mean(puc_dcovar_c(:,cas)<=level))

disp(' '),fprintf('  Empirical size (robust, deriving the kernel smoothed [h1,h2]) = %1.4f \n',mean(puc_dcovar_ck(:,cas)<=level))

disp(' '),fprintf('  == h1 violation test =='),disp(' ')

disp(' '),fprintf('  Empirical size h1 = %1.4f \n',mean(puc_h1(:,cas)<=level))

disp(' '),fprintf('  Empirical size h1 (With ER) = %1.4f \n',mean(puc_h1_risk(:,cas)<=level))

disp(' '),fprintf('  Empirical size h1 (robust, deriving the kernel smoothed [h1,h2]) = %1.4f \n',mean(puc_h1_ck(:,cas)<=level))

disp(' '),fprintf('  == h2 violation test =='),disp(' ')

disp(' '),fprintf('  Empirical size h2 = %1.4f \n',mean(puc_h2(:,cas)<=level))

disp(' '),fprintf('  Empirical size h2 (With ER) = %1.4f \n',mean(puc_h2_risk(:,cas)<=level))

disp(' '),fprintf('  Empirical size h2 (robust, deriving the kernel smoothed [h1,h2]) = %1.4f \n',mean(puc_h2_ck(:,cas)<=level))

end

% uc_H0=uc_dcovar;uc_risk_H0=uc_dcovar_risk;uc_c_kernel_H0=uc_dcovar_ck;
% 
% uc_h1_H0=uc_h1;uc_h1_risk_H0=uc_h1_risk;uc_h1_c_kernel_H0=uc_h1_ck;
% 
% uc_h2_H0=uc_h2;uc_h2_risk_H0=uc_h2_risk;uc_h2_c_kernel_H0=uc_h2_ck;
% 
% save Size_Marginal_DeltaCoVaR_forpowersizecorrected uc_H0    uc_risk_H0 uc_c_kernel_H0 ...
%                                uc_h1_H0 uc_h1_risk_H0 uc_h1_c_kernel_H0...
%                                uc_h2_H0 uc_h2_risk_H0 uc_h2_c_kernel_H0
