%---------------------------------------------------------------------------------------------------------------------------------
% PURPOSE: Compute the partial derivative of the expectation of ht(teta) function in the
% Bivariate Normal case with respect to variance(Y1), correlation(Y1,Y2) and variance(Y2)
%---------------------------------------------------------------------------------------------------------------------------------
% => Delta CoVaR backtesting case <=
%---------------------------------------------------------------------------------------------------------------------------------
% function [dEh1 , dEh2] = h_normal_derivatives_expectation_deltacovar(sigma_in,covar1_in,covar2_in,covar_options,z,alpha,btilde)
%
% Inputs:
%   sigma_in: Variance-Covariance matrix of the bivariate normal distribution
%   covar1_in: covar(alpha,alpha)
%   covar2_in: covar(alpha,beta)
%   covar_options: covar termination tolerance set for computation accuracy
%   z: Incremental value for derivatives
%   alpha: coverage rate for the VaR
%   btilde: Second coverage rate for the VaR
%
% Outputs:  dEh1 and dEh1 are (3,1) matrix
%  -  dEh1 => partial derivatives of the expectation of h1
%            * dEh1(1) : derivatives with respect to variance(Y1)
%            * dEh1(2) : derivatives with respect to correlation(Y1,Y2)
%            * dEh1(3) : derivatives with respect to variance(Y2)
%  -  dEh2 => partial derivatives of the expectation of h2
%            * Same construction than dEh1
% ---------------------------------------------------------------------- 
% J. Leymarie. December 1, 2016
% LEO, University of Orleans
% ----------------------------------------------------------------------

function [dEh1 , dEh2] = h_normal_derivatives_expectation_deltacovar(sigma_in,covar1_in,covar2_in,covar_options,z,alpha,btilde)
  
binf=btilde(1);bsup=btilde(2);                               % Two-sided Y2 truncation

VaR2alpha_in=norminv(alpha,0,sqrt(sigma_in(2,2)));           % Y2's VaR of order alpha

VaR2binf_in=norminv(binf,0,sqrt(sigma_in(2,2)));             % Y2's VaR of order binf            

VaR2bsup_in=norminv(bsup,0,sqrt(sigma_in(2,2)));             % Y2's VaR of order bsup

var1_in=sigma_in(1,1);                                       % Y1's variance

var2_in=sigma_in(2,2);                                       % Y2's variance

rho_in=sigma_in(1,2)/(sqrt(var1_in*var2_in));                % {Y1,Y2}'s correlation

theta_in=[var1_in;rho_in;var2_in];                           % Parameters for derivatives

dcovar1=NaN(3,1);                                            % CoVaR(alpha,alpha) derivative with respect to theta

dcovar2=NaN(3,1);                                            % "Median" CoVaR(alpha,btilde) derivative with respect to theta

dVaR2alpha=NaN(3,1);                                         % VaR2alpha derivative with respect to theta

dVaR2binf=NaN(3,1);                                          % VaR2binf derivative with respect to theta

dVaR2bsup=NaN(3,1);                                          % VaR2bsup derivative with respect to theta
    
%==========================
%=== Partial derivative ===
%==========================

for i=1:3

    theta_in_z=theta_in;                                     % Initialization of theta_in_z

    theta_in_z(i)=theta_in_z(i)+z;                           % z augmentation of the ith theta_in's element 

    var1_in_z=theta_in_z(1);                                 % z augmented Y1's variance                                     

    var2_in_z=theta_in_z(3);                                 % z augmented Y2's variance                                     

    rho_in_z=theta_in_z(2);                                  % z augmented {Y1,Y2}'s correlation
                                  
    cov_in_z=rho_in_z*sqrt(var1_in_z*var2_in_z);             % z augmented {Y1,Y2}'s covariance
     
    sigma_in_z=[var1_in_z cov_in_z; cov_in_z var2_in_z];     % z augmented {Y1,Y2}'s VCV matrix         

    VaR2alpha_z=norminv(alpha,0,sqrt(sigma_in_z(2,2)));      % z augmented Y2's VaR of order alpha

    VaR2binf_z=norminv(binf,0,sqrt(sigma_in_z(2,2)));        % z augmented Y2's VaR of order binf

    VaR2bsup_z=norminv(bsup,0,sqrt(sigma_in_z(2,2)));        % z augmented Y2's VaR of order bsup

    obj = @(x) (mvncdf([x;VaR2alpha_z],zeros(2,1),sigma_in_z)/normcdf(VaR2alpha_z,0,sqrt(sigma_in_z(2,2)))-alpha)^2;

    x0=0; covar1_z=fminsearch(obj,x0,covar_options);         % z augmented covar1                       

    obj = @(x) ((mvncdf([x;VaR2bsup_z],zeros(2,1),sigma_in_z)-mvncdf([x;VaR2binf_z],zeros(2,1),sigma_in_z))/...
        (normcdf(VaR2bsup_z,0,sqrt(sigma_in_z(2,2)))-normcdf(VaR2binf_z,0,sqrt(sigma_in_z(2,2))))-alpha)^2;

    x0=0; covar2_z=fminsearch(obj,x0,covar_options);         % z augmented covar2                         

    dcovar1(i)=(covar1_z-covar1_in)/z;                       % covar1 derivative with respect to theta_in

    dcovar2(i)=(covar2_z-covar2_in)/z;                       % covar2 derivative with respect to theta_in

    dVaR2alpha(i)=(VaR2alpha_z-VaR2alpha_in)/z;              % Y2's VaR of order alpha derivative with respect theta_in

    dVaR2binf(i)=(VaR2binf_z-VaR2binf_in)/z;                 % Y2's VaR of order binf derivative with respect to theta_in

    dVaR2bsup(i)=(VaR2bsup_z-VaR2bsup_in)/z;                 % Y2's VaR of order bsup derivative with respect to theta_in

end

% % To compare derivatives covar1_in and Owen & Chou covar1_in approximation :
% % dcovar_approx=[(covar_approx(var1_in+z,rho_in,alpha,alpha)-covar_approx(var1_in,rho_in,alpha,alpha))/z; ...
% %   (covar_approx(var1_in,rho_in+z,alpha,alpha)-covar_approx(var1_in,rho_in,alpha,alpha))/z;0];
% % disp([dcovar1 dcovar_approx])

% cdf derivatives for r1covart quantity
dF1_r1=(mvncdf([covar1_in+z VaR2alpha_in],0,sigma_in)- ...   % 1st cdf derivatives used to build h1 expectation derivative
    mvncdf([covar1_in VaR2alpha_in],0,sigma_in))/z;

dF2_r1=(mvncdf([covar1_in VaR2alpha_in+z],0,sigma_in)- ...   % 2st cdf derivatives used to build h1 expectation derivative
    mvncdf([covar1_in VaR2alpha_in],0,sigma_in))/z;

% cdf derivatives for r2covart quantity
dF1_r2=(mvncdf([covar2_in+z VaR2bsup_in],0,sigma_in)- ...    % 1st cdf derivatives used to build h2 expectation derivative
    mvncdf([covar2_in VaR2bsup_in],0,sigma_in))/z;

dF2_r2=(mvncdf([covar2_in VaR2bsup_in+z],0,sigma_in)- ...    % 2nd cdf derivatives used to build h2 expectation derivative
    mvncdf([covar2_in VaR2bsup_in],0,sigma_in))/z;

dF3_r2=(mvncdf([covar2_in+z VaR2binf_in],0,sigma_in)- ...    % 3th cdf derivatives used to build h2 expectation derivative
    mvncdf([covar2_in VaR2binf_in],0,sigma_in))/z;

dF4_r2=(mvncdf([covar2_in VaR2binf_in+z],0,sigma_in)- ...    % 4th cdf derivatives used to build h2 expectation derivative
    mvncdf([covar2_in VaR2binf_in],0,sigma_in))/z;

dEh1=dcovar1*dF1_r1+dVaR2alpha*dF2_r1;                       % h1 expectation's derivative with respect to theta_in

dEh2=dcovar2*dF1_r2+dVaR2bsup*dF2_r2- ...                    % h2 expectation's derivative with respect to theta_in
    dcovar2*dF3_r2-dVaR2binf*dF4_r2;                          
                                                                         
end