%------------------------------------------------------------------------------
% PURPOSE: "Backtesting Systemic Risk Measures", Banulescu, Leymarie, Hurlin and Scaillet (2016)
% Monte Carlo Simulations based on a Multivariate Normal Distribution with estimation risk
% -----------------------------------------------------------------------------
%   H1 : Empirical Power for 3 configuations (H1A, H1B, H1C)
% ---------------------------------------------------------------------
% J. Leymarie, November 27, 2017.
% LEO, University of Orleans
% ---------------------------------------------------------------------
% Note: The calibration of the parameters is obtained from the estimation 
%       of a GJR-DCC model on BAC and CRSP market weighted index (2012-2016 included)
% ---------------------------------------------------------------------

clear ; clc ; close all ;  tic

help Main_Power_Marginal_MES

load Size_Marginal_MES_forpowersizecorrected

%==================
%=== Parameters ===
%==================

cases=[250 250 ; 250 500 ; 500 250 ; 500 500; 2500 250; 2500 500];    % Sample sizes (first colum T, second coum N)

S=10000;                                                              % Number of replications

alpha=0.05;                                                           % alpha level for quantile of Y2

level=0.05;                                                           % Nominal size of the tests

mmax=5;                                                               % Maximum lag autocovariance for IND test

options=optimset('Display','off');                                    % Options

%================================
%=== Variance-Covariance Parameters ===
%================================

a1=[0.149;0.065;0.075;0.855];                      % GARCH parameters for Y1 (GJR-DCC)

a2=[0.057;0.001;0.268;0.786];                      % GARCH parameters for Y2 (GJR-DCC)
                 
b=[0.663;0.041;0.754];                             % DCC parameters (rho_bar,a,b) (Normal DCC) 
                         
var1=a1(1)/(1-a1(2)-0.5*a1(3)-a1(4));              % Unconditional variance for Y1

var2=a2(1)/(1-a2(2)-0.5*a2(3)-a2(4));              % Unconditional variance for Y2

corr12=b(1);                                       % Unconditional correlation (Normal DCC)

cov12=corr12*sqrt(var1)*sqrt(var2);                % Unconditional covariance

Sigma=[var1 cov12 ; cov12 var2];                   % Unconditional variance-covariance matrix

VaR2=sqrt(var2)*norminv(alpha);                    % Unconditional Value-at-Risk of level alpha for Y2

%=====================
%=== Configurations H1 ===
%=====================

factor=[0.75 0.5 0.25 ;...                        % Decreasing Factors for H1 : first line: var1, second line :var2, third line: correlation
        0.75 0.5 0.25 ;...
        0.75 0.5 0.25]; 

% factor=[0.9 0.8 0.7 ;...                        % Decreasing Factors for H1 : first line: var1, second line :var2, third line: correlation
%         0.9 0.8 0.7 ;...
%         0.9 0.8 0.7]; 

nfactor=numel(factor);                            % Number of H1 hypothesis considered

%================
%=== Initialization ===
%================

ncases=size(cases,1);                             % Number of considered cases

uc=NaN(S,nfactor,ncases);                         % UC statistics without estimation risk            

ind=NaN(S,mmax,nfactor,ncases);                   % IND statistics without estimation risk            

uc_risk=uc; ind_risk=ind;                         % UC and IND statistics with estimation risk

ind_c=ind; ind_c_kernel=ind;                      % IND robust statistics

uc_c=uc; uc_c_kernel=uc;                          % UC robust statistics

for cas=1:ncases                                  % Loop on the sizes                             

    T=cases(cas,1);                                                       % In sample size

    N=cases(cas,2);                                                       % Out of sample size

    TT=T+N;                                                               % Total size

    %============================
    %=== Simulations under H0 ===
    %============================

    for h_conf=1:1:numel(factor)                                          % Loop on the configuations H1A, H1B, H1C             

    parfor s=1:S                                                          % Loop for the simulations

        epsilon=mvnrnd(zeros(2,1),eye(2),TT);                             % Simulated innovations 

        Y=epsilon*chol(Sigma);                                            % Simulated series

        %========================
        %=== Estimation In-Sample ===
        %========================

        Sigma_in_H0=cov(Y(1:T,:),1);                                      % Estimated unconditional variance-covariance matrix 
        
        theta_in_H0=[Sigma_in_H0(1,1) ...                                 % Estimated parameters in-sample
                  Sigma_in_H0(1,2)/sqrt(Sigma_in_H0(2,2)*Sigma_in_H0(1,1))...    
                  Sigma_in_H0(2,2)]';                   
 
        if h_conf<=size(factor,2)                                         % H1A: error in variance1 estimate

            var1_H1=Sigma(1,1)*factor(1,h_conf);                          % H1A: error in variance1 estimate

            theta_in=[var1_H1 ; theta_in_H0(2:end)];                      % Constrained estimates for H1A

            Mask=[0 1 0 ; 0 0 1];                                         % Mask matrix allows to delete the first row of MES (variance1)

        elseif (h_conf>size(factor,2))&&(h_conf<=2*size(factor,2))        % H1B: error in variance2 estimate

            var2_H1=Sigma(2,2)*factor(2,h_conf-size(factor,2));           % H1B: error in variance2 estimate

            theta_in=[theta_in_H0(1:2) ; var2_H1 ];                       % Constrained estimates for H1B

            Mask=[1 0 0 ; 0 1 0];                                         % Mask matrix allows to delete the third row of MES (variance2)

        elseif h_conf>2*size(factor,2)                                    % H1C: error in correlation estimate

            rho_H1=Sigma(1,2)/sqrt(prod(diag(Sigma)))*factor(3,h_conf-2*size(factor,2));   % H1C: error in correlation estimate                

            theta_in=[theta_in_H0(1) ; rho_H1; theta_in_H0(3)];           % Constrained estimates for H1C

            Mask=[1 0 0 ; 0 0 1];                                         % Mask matrix allows to delete the second row of MES (correlation)                

        end                                                               % End of loop on h_conf 

        Sigma_in=[theta_in(1)  theta_in(2)*sqrt(theta_in(1)*theta_in(3)) ;...
                  theta_in(2)*sqrt(theta_in(1)*theta_in(3)) theta_in(3)];

        var1_in=Sigma_in(1,1);                                            % Estimated unconditional variance for Y1

        var2_in=Sigma_in(2,2);                                            % Estimated unconditional variance for Y1

        rho_in=Sigma_in(1,2)/sqrt(var2_in*var1_in);                       % In sample correlation

        cov_in=Sigma_in(1,2);                                             % In sample covariance

        VaR2_in=sqrt(var2_in)*norminv(alpha);                             % Estimated Value-at-Risk(alpha) for Y2

        Hessian=hessian_2sided('loglikelihood_normal',...                 % Hessian matrix under the null (Loglik is the opposite of the likelihood)
                theta_in_H0,Y(1:T,:));                                    % Estimation error made in H1 is supposed missed by the estimation procedure   
                                                                          
        V_asy=Mask*pinv(Hessian)*Mask';                                   % Asymptotic variance-covariance of ML estimators

        %==========================
        %=== Forecasts out-of-sample ===
        %==========================

        var1_out=var1_in;                                                 % Unconditional variances forecasts for Y1

        var2_out=var2_in;                                                 % Unconditional variances forecasts for Y2

        rho_out=rho_in;                                                   % Unconditional correlation forecast 

        Sigma_out=Sigma_in;                                               % Unconditional variance covariance matrix

        VaR2_out=VaR2_in;                                                 % Unconditional VaR forecasts

        Y1_out=Y(T+1:end,1);                                              % OOS simulated realisations for Y1

        Y2_out=Y(T+1:end,2);                                              % OOS simulated realisations for Y2

        Y_out=[Y1_out Y2_out];                                            % OOS simulated realisations for [Y1 Y2]

        %=================================================
        %=== Computation of H(alpha) series: without estimation risk  ===
        %=================================================

        Y1_out_s=Y1_out(Y2_out<=VaR2);                                                 % Value of Y1 for which Y<VaR2

        u12=mvncdf([Y1_out_s VaR2*ones(size(Y1_out_s))],zeros(1,2),Sigma)/alpha;       % u12 series

        H=zeros(N,1);                                                                  % Ht(alpha) series

        H(Y2_out<=VaR2)=1-u12;                                                         % Ht(alpha) series (evaluated for Y1 for which Y2<VaR2)

        %==============================================
        %=== Computation of H(alpha) series: with estimation risk  ===
        %==============================================

        Y1_out_risk=Y1_out(Y2_out<=VaR2_out);                                          % Value of Y1 for which Y<VaR2_out           

        u12=mvncdf([Y1_out_risk VaR2_out*ones(size(Y1_out_risk))],0,Sigma_out)/alpha;  % u12 series with estimation risk

        H_risk=zeros(N,1);                                                             % Ht(alpha) series

        H_risk(Y2_out<=VaR2_out)=1-u12;                                                % Ht(alpha) series (evaluated for Y1 for which Y2<VaR2

        %=================================
        %== UC_MES - Unconditionnal backtest ===  
        %=================================

        % Without Estimation Risk

       uc(s,h_conf,cas)=sqrt(N)*(mean(H)-alpha/2)/sqrt(alpha*(1/3-alpha/4));           % Statistic UC without estimation risk

        % With Estimation Risk

       uc_risk(s,h_conf,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)); % Statistic UC with estimation risk

        % Robust Test Statistics

        RMESt_kernel=Mask*H_normal_derivatives_kernel(Sigma_out,Y_out,alpha,1/N);      % Correction term RMES (H kernel numerical derivative)

        RMES_kernel=mean(RMESt_kernel,2);                                              % RMES mean

        uc_c_kernel(s,h_conf,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)+...  % Statistic UC robust
            N/T*RMES_kernel'*T*V_asy*RMES_kernel);   

        RMES=Mask*[RMES_CoVaR_term(var1_out,var2_out,rho_out,VaR2_out,1,alpha);...     % Correction term RMES (CoVaR methodology)
            RMES_CoVaR_term(var1_out,var2_out,rho_out,VaR2_out,2,alpha);...
            RMES_VaR2_term(var1_out,var2_out,rho_out,VaR2_out,3,alpha)];

        uc_c(s,h_conf,cas)=sqrt(N)*(mean(H_risk)-alpha/2)/sqrt(alpha*(1/3-alpha/4)+... % Statistic UC robust
                N/T*RMES'*T*V_asy*RMES);                             

        %=================================
        %=== IND_MES - Conditionnal backtest  ===
        %=================================

        gammaHt_hat=NaN(mmax,1);gammaHt_hat_risk=gammaHt_hat;                           % Autocovariances of Ht(alpha)

        rhoHt_hat=NaN(mmax,1);rhoHt_hat_risk=rhoHt_hat;                                 % Autocorrelations of Ht(alpha)

        R_kernel=NaN(mmax,2);                                                           % Correction term R (Ht kernel numerical derivative)  

        R=NaN(mmax,2);                                                                  % Correction term R (CoVaR methodology)                            

        for m=1:mmax                                                                         % Loop on the lag

            % Autocorrelations with and without estimation risk

            Hlag=[NaN(m,1) ; H(1:N-m)];                                                      % Lagged values

            gammaHt_hat(m)=(1/(N-m))*nansum((H-alpha/2).*(Hlag-alpha/2));                    % Autocovariances of H(t)

            rhoHt_hat(m)=gammaHt_hat(m)/var(H);                                              % Autocorrelations of H(t)

            Hlag_risk=[NaN(m,1) ; H_risk(1:N-m)];                                            % Lagged values with Estimation Risk

            gammaHt_hat_risk(m)=(1/(N-m))*nansum((H_risk-alpha/2).*(Hlag_risk-alpha/2));     % Autocovariances of H(t) with Estimation Risk

            rhoHt_hat_risk(m)=gammaHt_hat_risk(m)/var(H_risk);                               % Autocorrelations of H(t) with Estimation Risk

            % Matrix Ri and Rj 

            R_kernel(m,:)=(1/(alpha*(1/3-alpha/4)))*(nansum(repmat((Hlag_risk-alpha/2),1,2).*RMESt_kernel')/(N-m));  % Be careful: RMES must be a vector in the conditional case, do not use repmat

            R(m,:)=(1/(alpha*(1/3-alpha/4)))*(nansum(repmat((Hlag_risk-alpha/2),1,2).*repmat(RMES',N,1))/(N-m));     % Be careful: RMES must be a vector in the conditional case, do not use repmat

            % Without Estimation Risk

            ind(s,m,h_conf,cas)=N*sum(rhoHt_hat(1:m).^2);                                     % Statistic IND without estimation risk (Box-Pierce)

            % With estimation risk

            ind_risk(s,m,h_conf,cas)=N*sum(rhoHt_hat_risk(1:m).^2);                           % Statistic IND with estimation risk 

            % Robust test statistics

            delta=eye(m)+N*R(1:m,:)*V_asy*R(1:m,:)';                                          % Variance-covariance matrix of the corrected statistic

            delta_kernel=eye(m)+N*R_kernel(1:m,:)*V_asy*R_kernel(1:m,:)';                     % Variance-covariance matrix of the corrected statistic (kernel)

            ind_c_kernel(s,m,h_conf,cas)=N*rhoHt_hat_risk(1:m)'*pinv(delta_kernel)*rhoHt_hat_risk(1:m);   % Statistic IND robust (Ht kernel numerical derivative)

            ind_c(s,m,h_conf,cas)=N*rhoHt_hat_risk(1:m)'*pinv(delta)*rhoHt_hat_risk(1:m);                 % Statistic IND robust (CoVaR methodology)

        end                                                                                   % Loop on the lag order m

    end                                                                   % Loop on S
      
    disp('==================================================================================='),disp(' ')

    if h_conf<=size(factor,2)                                             % H1A: error in variance1 estimate

        fprintf('  Hypothesis H1A: decrease in variance(Y1) by %1.4f \n',1-factor(1,h_conf)),disp(' ')

    elseif (h_conf>size(factor,2))&&(h_conf<=2*size(factor,2))            % H1B: error in variance2 estimate

        fprintf('  Hypothesis H1B: decrease in variance(Y2) by %1.4f \n',1-factor(2,h_conf-size(factor,2))),disp(' ')            

    elseif h_conf>2*size(factor,2)                                        % H1C: error in correlation estimate

        fprintf('  Hypothesis H1C: decrease in correlation by %1.4f \n',1-factor(3,h_conf-2*size(factor,2))),disp(' ')                

    end                                                                   % End of loop on h_conf 

    fprintf('  In-sample size = %1.0f    Out-of-sample size = %1.0f   Number of replications = %1.0f \n',T,N,S),disp(' ')

    fprintf('  Coverage rate for VaR = %1.3f      Size = %1.3f \n',alpha,level),disp(' ')

    fprintf('  Max lag (for IND test) = %1.0f  \n',mmax)

    disp(' '),disp('  ==== Test UC ====')

    disp(' '),fprintf('  Empirical power (with ER) = %1.4f \n',mean(abs(uc_risk(:,h_conf,cas))>=quantile(abs(uc_risk_H0(:,cas)),1-level)))      

    disp(' '),fprintf('  Empirical power (robust, Kernel method) = %1.4f \n',mean(abs(uc_c_kernel(:,h_conf,cas))>=quantile(abs(uc_c_kernel_H0(:,cas)),1-level)))  

    disp(' '),fprintf('  Empirical power (robust, CoVaR method) = %1.4f \n',mean(abs(uc_c(:,h_conf,cas))>=quantile(abs(uc_c_H0(:,cas)),1-level)))

    disp(' '), disp('  ==== Test IND ====')

    affi=5;

    disp(' '),disp('  Lags ')  

    disp(sprintf('   %1.4f ',(1:1:affi)) )

    disp(' ');disp('  Empirical power (with ER)')

    disp(sprintf('   %1.4f ',mean(abs(ind_risk(:,1:affi,h_conf,cas))>=repmat(quantile(abs(ind_risk_H0(:,1:affi,cas)),1-level),S,1))))   

    disp(' ');disp('  Empirical power (robust, Kernel method)')

    disp(sprintf('   %1.4f ',mean(abs(ind_c_kernel(:,1:affi,h_conf,cas))>=repmat(quantile(abs(ind_c_kernel_H0(:,1:affi,cas)),1-level),S,1))))   

    disp(' ');disp('  Empirical power (robust, CoVaR method)')

    disp(sprintf('   %1.4f ',mean(abs(ind_c(:,1:affi,h_conf,cas))>=repmat(quantile(abs(ind_c_H0(:,1:affi,cas)),1-level),S,1))))   

    disp(' ')

    end                                                   % End of loop on the H1 alternatives
    
end                                                       % End of loop on the cases

toc
